@@ -33,16 +33,13 @@ impl From<ResetMode> for u8 {
     }
 }
 
-/// Performs an ECU Reset operation
-///
-/// ## Params
-/// * server - KWP2000 diagnostic server
-/// * mode - [ResetMode] to send to the ECU
-pub fn execute_reset(
-    server: &mut Kwp2000DiagnosticServer,
-    mode: ResetMode,
-) -> DiagServerResult<()> {
-    server
-        .execute_command_with_response(KWP2000Command::ECUReset, &[mode.into()])
-        .map(|_| ())
+impl Kwp2000DiagnosticServer {
+    /// Performs an ECU Reset operation
+    ///
+    /// ## Params
+    /// * mode - [ResetMode] to send to the ECU
+    pub fn reset_ecu(&mut self, mode: ResetMode) -> DiagServerResult<()> {
+        self.execute_command_with_response(KWP2000Command::ECUReset, &[mode.into()])
+            .map(|_| ())
+    }
 }
