@@ -53,16 +53,14 @@ impl From<SessionType> for u8 {
     }
 }
 
-/// Sets the ECU into a diagnostic mode
-///
-/// ## Parameters
-/// * server - The KWP2000 Diagnostic server
-/// * mode - The [SessionType] to put the ECU into
-pub fn set_diagnostic_session_mode(
-    server: &mut Kwp2000DiagnosticServer,
-    mode: SessionType,
-) -> DiagServerResult<()> {
-    server
-        .execute_command_with_response(KWP2000Command::StartDiagnosticSession, &[mode.into()])
-        .map(|_| ())
+impl Kwp2000DiagnosticServer {
+    /// Sets the ECU into a diagnostic mode
+    ///
+    /// ## Parameters
+    /// * server - The KWP2000 Diagnostic server
+    /// * mode - The [SessionType] to put the ECU into
+    pub fn set_diagnostic_session_mode(&mut self, mode: SessionType) -> DiagServerResult<()> {
+        self.execute_command_with_response(KWP2000Command::StartDiagnosticSession, &[mode.into()])
+            .map(|_| ())
+    }
 }
