@@ -1,9 +1,19 @@
-#![deny(missing_docs, missing_debug_implementations)]
-#![allow(dead_code)]
+#![deny(
+    missing_docs,
+    missing_debug_implementations,
+    missing_copy_implementations,
+    trivial_casts,
+    trivial_numeric_casts,
+    unstable_features,
+    unused_imports,
+    unused_import_braces,
+    unused_qualifications
+)]
 
-//! A crate which provides the most common ECU diagnostic protocols used by modern ECUs in vehicles.
+//! A crate which provides the most common ECU diagnostic protocols used by modern ECUs in vehicles,
+//! as well as common hardware APIs for accessing and using diagnostic adapters
 //!
-//! ## Protocol support
+//! ## ECU Diagnostic protocol support
 //!
 //! This crate provides the 3 most widely used diagnostic protocols used by modern ECUs from 2000 onwards
 //!
@@ -13,7 +23,6 @@
 //! OBD2 is designed to be safe and simple, and does not write data to the ECU.
 //!
 //!
-//!
 //! ### Keyword protocol 2000 (KWP2000)
 //! ISO14230 - KWP2000 is a advanced diagnostic protocol utilized by many vehicle manufacturers from 2000-2006 (Superseded by UDS).
 //! Unlike OBD2, KWP2000 allows for much more complex operations, which could potentially cause damage to a vehicle if used incorrectly.  
@@ -24,7 +33,8 @@
 //! * Low level manipulation of ECU's EEPROM or RAM
 //! * Gateway access in vehicles which have them
 //!
-//! The specification implemented in this crate is v2.2, dated 05-08-2002
+//! The specification implemented in this crate is v2.2, dated 05-08-2002.
+//!
 //!
 //! ### Unified diagnostic services (UDS)
 //! ISO14429 - UDS is an advanced diagnostic protocol utilized by almost all vehicle manufacturers from 2006 onwards. Like KWP2000,
@@ -32,28 +42,36 @@
 //!
 //! The specification implemented in this crate is the second edition, dated 01-12-2006.
 //!
-//! ## Usage
-//! In order to utilize any of the diagnostic servers, you will need
-//! to implement the [channel::PayloadChannel] trait, which allows for the diagnostic servers
-//! to send and receive data to/from the ECU, regardless of the transport layer used.
+//! ## Hardware support (VCIs)
+//!
+//! This crate provides support for the following VCI adapters and hardware protocols, as well as a convenient interface
+//! for making your own adapter API for customized hardware
+//!
+//! ### SocketCAN (Linux only)
+//! This crate provides support for socketCAN compatible adapters, for utilizing both ISO-TP and regular CAN communication
+//!
+//! ### SAE J2534-2
+//!
+//! SAE J2534 (AKA Passthru) is a VCI adapter protocol which allows a VCI to communicate with a vehicle using multiple various
+//! different network types, including CAN, ISO-TP, J1850, ISO9141 and SCI.
+//!
+//! NOTE: Although the J2534 API is officially only for Windows, it will also compile for UNIX and OSX operating
+//! systems, due to the unofficial porting of the API in the [Macchina-J2534 project](https://github.com/rnd-ash/Macchina-J2534)
 //!
 
-pub use channel::ChannelError;
-pub use hardware::HardwareError;
+use channel::ChannelError;
+use hardware::HardwareError;
 
 pub mod channel;
 pub mod dtc;
+pub mod dynamic_diag;
 pub mod hardware;
 pub mod kwp2000;
 pub mod obd2;
 pub mod uds;
-pub mod dynamic_diag;
 
 mod helpers;
 
-/// Version of compiled crate
-pub const CRATE_VERSION: &str = env!("CARGO_PKG_VERSION");
-
 /// Diagnostic server result
 pub type DiagServerResult<T> = Result<T, DiagError>;
 
@@ -63,11 +81,11 @@ pub enum DiagError {
     /// The Diagnostic server does not support the request
     NotSupported,
     /// Diagnostic error code from the ECU itself
-    ECUError{
+    ECUError {
         /// Raw Negative response code from ECU
-        code: u8, 
+        code: u8,
         /// Negative response code definition according to protocol
-        def: Option<String>
+        def: Option<String>,
     },
     /// Response empty
     EmptyResponse,
@@ -87,19 +105,21 @@ pub enum DiagError {
     NotImplemented(String),
     /// Device hardware error
     HardwareError(HardwareError),
+    /// ECU Param ID did not match the request, but the Service ID was correct
+    MismatchedResponse(String),
 }
 
 impl std::fmt::Display for DiagError {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         match &self {
             DiagError::NotSupported => write!(f, "request not supported"),
-            DiagError::ECUError{code, def } => {
+            DiagError::ECUError { code, def } => {
                 if let Some(d) = def {
                     write!(f, "ECU error 0x{:02X} ({})", code, d)
                 } else {
                     write!(f, "ECU error 0x{:02X}", code)
                 }
-            },
+            }
             DiagError::EmptyResponse => write!(f, "ECU provided an empty response"),
             DiagError::WrongMessage => write!(f, "ECU response message did not match request"),
             DiagError::ServerNotRunning => write!(f, "diagnostic server not running"),
@@ -111,7 +131,8 @@ impl std::fmt::Display for DiagError {
             DiagError::NotImplemented(s) => {
                 write!(f, "server encountered an unimplemented function: {}", s)
             }
-            &DiagError::HardwareError(e) => write!(f, "Hardware error: {}", e),
+            DiagError::HardwareError(e) => write!(f, "Hardware error: {}", e),
+            DiagError::MismatchedResponse(e) => write!(f, "Param mismatched response: {}", e),
         }
     }
 }
@@ -158,7 +179,7 @@ pub enum ServerEvent<'a, SessionState> {
         new: SessionState,
     },
     /// Received a request to send a payload to the ECU
-    Request(&'a[u8]),
+    Request(&'a [u8]),
     /// Response from the ECU
     Response(&'a DiagServerResult<Vec<u8>>),
     /// An error occurred whilst transmitting tester present message
@@ -181,11 +202,14 @@ pub trait ServerEventHandler<SessionState>: Send + Sync {
 
 /// Base trait for diagnostic servers
 pub trait DiagnosticServer<CommandType> {
-
     /// Sends a command to the ECU, and doesn't poll for its response
     fn execute_command(&mut self, cmd: CommandType, args: &[u8]) -> DiagServerResult<()>;
     /// Sends a command to the ECU, and polls for its response
-    fn execute_command_with_response(&mut self, cmd: CommandType, args: &[u8]) -> DiagServerResult<Vec<u8>>;
+    fn execute_command_with_response(
+        &mut self,
+        cmd: CommandType,
+        args: &[u8],
+    ) -> DiagServerResult<Vec<u8>>;
     /// Sends an arbitrary byte array to the ECU, and doesn't poll for its response
     fn send_byte_array(&mut self, bytes: &[u8]) -> DiagServerResult<()>;
     /// Sends an arbitrary byte array to the ECU, and polls for its response
@@ -222,3 +246,22 @@ pub trait BaseServerPayload {
     /// Boolean indicating if the diagnostic server should poll the ECU for a response after sending the payload
     fn requires_response(&self) -> bool;
 }
+
+/// Converts a single byte into a BCD string
+pub fn bcd_decode(input: u8) -> String {
+    format!("{}{}", (input & 0xF0) >> 4, input & 0x0F)
+}
+
+/// Converts a slice to a BCD string
+pub fn bcd_decode_slice(input: &[u8], sep: Option<&str>) -> String {
+    let mut res = String::new();
+    for (pos, x) in input.iter().enumerate() {
+        res.push_str(bcd_decode(*x).as_str());
+        if let Some(separator) = sep {
+            if pos != input.len() - 1 {
+                res.push_str(separator)
+            }
+        }
+    }
+    res
+}
